access_by_lua_block {
    local this_ip = ngx.var.remote_addr
    local this_x_client = ngx.var.http_x_client

    -- LUA rate-limit
    local do_rate_limit = true
    local whitelist = {
        "89.160.32.195"
    }

    -- This is the same as in aws/nginx.conf
    -- Reverted to old solution until we can properly fix the new one.
    local resolver_ip = "SET_RESOLVER"

    for k, v in pairs(whitelist) do
        local ip_whitelist = string.find(this_ip, v)
        if ip_whitelist then
            do_rate_limit = false
            break
        end
    end

    if do_rate_limit then
        local md5 = require "md5"
        local json  = require 'cjson'
        local request = require "resty.rate.limit"

        if this_user_agent == nil then
            local req_key = this_ip
        else
            local req_key = md5.sumhexa(this_ip .. this_user_agent)
        end

        request.limit {
            key = req_key,
            rate = 1000,
            interval = 10,
            long_interval = 3600,
            log_level = ngx.ERR,
            redis_config = { host = os.getenv("REDIS_FC_HOST"), port = 6379, timeout = 1, pool_size = 100 }
        }

        -- LUA rDNS lookup
        -- If the ngx.var.check_auth is true, proceed with rDNS check
        if ngx.var.http_user_agent == nil then
            ngx.log(ngx.ERR, "RESTY DNS EMPTY UA from: " .. this_ip)
            ngx.exit(400) -- return with a 400 (bad request)
        end

        local this_user_agent = string.lower(ngx.var.http_user_agent)
        local lookup = ngx.shared.lookup
        local has_ip = lookup:get(this_ip)
        local bots = {
            googlebot={ '.google.com', '.googlebot.com' },
            bingbot={ '.search.msn.com' },
            baiduspider={ '.crawl.baidu.com', '.crawl.baidu.jp' },
            yandex={ 'yandex.ru', 'yandex.net', 'yandex.com' },
            semrushbot={ '.semrush.com' },
            pingdom={ '.pingdom.com' },
            trendictionbot={ '.trendiction.de' },
            applebot={ '.applebot.apple.com' },
            ahrefbot={ '.ip-54-36-148.eu', '.ip-54-36-149.eu', '.ip-54-36-150.eu' },
            caam={ '.localhost' }
        }

        -- Before running the DNS lookup, check if this_ip has been through before
        if has_ip == "true" then
            return
        elseif has_ip == "false" then
            ngx.exit(403)
        else
            -- Perform reverse lookup if UA match
            for k,v in pairs(bots) do
                if this_user_agent ~= nil and this_user_agent ~= '' and string.find(this_user_agent, k) then
                    allow = false
                    local ua = k

                    local resolver = require "resty.dns.resolver"
                    local r, err = resolver:new{
                        nameservers = {resolver_ip},
                        retrans = 5,  -- 5 retransmissions on receive timeout
                        timeout = 2000,  -- 2 sec
                    }

                    if not r then
                        ngx.log(ngx.ERR, "RESTY DNS failed to instantiate the resolver: ", err)
                        return
                    end

                    local ptr_record = resolver.arpa_str(this_ip)
                    local answer, err = r:query(ptr_record, { qtype = r.TYPE_PTR })

                    if answer then
                        if not answer.errcode then
                            local ptrdname = answer[1].ptrdname

                            if ptrdname ~= nil then
                                for i,v in pairs(v) do
                                    local start,stop = string.find(ptrdname, v)
                                    local len = string.len(ptrdname)

                                    if stop == len then
                                        allow = true
                                    end
                                end
                            else
                                ngx.log(ngx.ERR, "RESTY DNS ptrdname was nil: ", json.encode(answer))
                                return
                            end

                            if allow == true then
                                -- Request is allowed, add to local cache permanently
                                -- and proceed with the request
                                local succ, err = lookup:add(this_ip, "true")
                                return
                            end

                            -- Request not allowed, add to local cache and deny the request
                            local succ, err = lookup:add(this_ip, "false", 600)
                            ngx.log(ngx.ERR, "RESTY DNS DENY from: " .. k .. " | " .. this_ip .. " | " .. ptrdname .. " | " .. this_user_agent)
                            ngx.exit(403)
                        else
                            ngx.log(ngx.ERR, "RESTY DNS server returned error code: ", answer.errcode, ", ", answer.errstr)
                            return
                        end
                    else
                        ngx.log(ngx.ERR, "RESTY DNS failed to query: ", err)
                        return
                    end
                end
            end
        end
    end
}
